from __future__ import annotations
import os
from dataclasses import dataclass
from ..helpers.file.file_helper import FileHelper
from ..models.problem import Problem


@dataclass
class ProblemGenerator:
    problem: Problem

    def read_template(self) -> ProblemGenerator:
        self.template = FileHelper.read_template(
            provider=self.problem.provider, language=self.problem.language
        )
        return self

    def build_path(self, problem_set: str) -> ProblemGenerator:
        self.problem.problem_set = problem_set
        self.problem.path = os.path.join(
            self.problem.provider.name, problem_set, self.problem.name
        )
        return self

    def generate_solution(self) -> ProblemGenerator:
        print("generating solution")
        print(self.problem)
        # {problem_name}
        # {problem_number}
        # {total_problems}
        # {site_name}
        # {contest_name}
        solution = FileHelper.template_filler(
            self.template,
            self.problem.name,
            str(1),
            str(1),
            self.problem.provider.name,
            self.problem.problem_set,
        )
        # creating the solution
        FileHelper.write_file(
            self.problem.path,
            f"main.{self.problem.language.ext}",
            file=solution,
        )
        return self

    def generate_input(self) -> ProblemGenerator:
        FileHelper.write_file(
            self.problem.path,
            "main.in",
            file="",
        )
        return self



from __future__ import annotations
import subprocess
import shlex


class SingleProcess:
    def __init__(self, *args) -> None:
        self.command = " ".join(
            [arg if isinstance(arg, str) else str(arg) for arg in args]
        )

    def run(self) -> SingleProcess:
        self.proc = subprocess.Popen(
            shlex.split(self.command),
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            bufsize=1,
            universal_newlines=True,
        )
        return self

    def communicate(self, input: str = "") -> tuple[str, str]:
        stdout, stderr = self.proc.communicate(input=input)
        return self.stringify(stdout), self.stringify(stderr)

    @staticmethod
    def stringify(buffer) -> str:
        if buffer is None:
            return ""
        res = ""
        for line in buffer:
            res += line
        return res



from __future__ import annotations
import sys

sys.setrecursionlimit(2 * 10**5)

## problem {} ({}/{})
## {}, {}
## author: @YOUR_NAME
## generated by contest-builder
## https://github.com/shamir0xe/contest-builder


# define variables
n: int = 0
some_list: list[int]


# define other functions here
def fn():
    return (n**2,)


class Mediator:
    def read_input(self) -> Mediator:
        global n, some_list
        readline = sys.stdin.readline

        n = int(readline().rstrip())
        some_list = list(map(int, readline().split()))
        return self

    def solve(self):
        """
        main logic comes here
        """
        global n, some_list

        return (fn(),)


def main():
    mediator = Mediator().read_input()
    ans = mediator.solve()
    print(" ".join(list(map(str, (ans,)))))


if __name__ == "__main__":
    main()



from __future__ import annotations
from ..helpers.config.local_config import LocalConfig
from ..generators.problem_generator import ProblemGenerator
from ..models.problem import Problem
from ..helpers.terminal.arg_helper import ArgHelper


class ProblemMediator:
    def read_configs(self) -> ProblemMediator:
        self.attributes = Problem.modifiables()
        self.problem_data = LocalConfig.read("problem")
        return self

    def read_args(self) -> ProblemMediator:
        self.problem_data = (
            ArgHelper(attributes=self.attributes, data=self.problem_data, prefix="--")
            .check_help()
            .check_args(skipps=["problem"])
            .must_include("name")
            .get_data()
        )
        return self

    def generate(self) -> ProblemMediator:
        problem = Problem.from_dict(self.problem_data)
        print(f"generating the following problem: {problem}")
        ProblemGenerator(problem=problem).read_template().build_path(
            problem_set=LocalConfig.read("problemset.folder.name")
        ).generate_solution().generate_input()
        return self

    def closure(self) -> ProblemMediator:
        print("DONE")
        return self



from __future__ import annotations
import sys

sys.setrecursionlimit(2 * 10**5)
# input = io.BytesIO(os.read(0, os.fstat(0).st_size)).readline

## problem {} ({}/{})
## {}, {}
## author: @YOUR_NAME
## generated by contest-builder
## https://github.com/shamir0xe/contest-builder


# define variables
n: int = 0
some_list: list[int]


# define other functions here
def fn():
    return (n**2,)


class Mediator:
    def read_input(self) -> Mediator:
        global n, some_list
        readline = sys.stdin.readline

        n = int(readline().rstrip())
        some_list = list(map(int, readline().split()))
        return self

    def solve(self) -> Mediator:
        """
        main logic comes here
        """
        global n, some_list

        self.ans = (fn(),)
        return self

    def print_output(self) -> None:
        print(" ".join(list(map(str, self.ans))))


def main(test_case: bool = False):
    def main_part():
        Mediator().read_input().solve().print_output()

    if test_case:
        t = int(input())
        for _ in range(t):
            main_part()
    else:
        main_part()


if __name__ == "__main__":
    main(False)



from ..helpers.model.finders.language_finder import LanguageFinder
from ..models.language import Language
from ..models.problem import Problem
from pylib_0xe.file.file import File


class ProblemActions:
    @staticmethod
    def set_problem_language(problem: Problem) -> Language:
        all_files = File.get_all_files(directory=problem.path)
        for filename in all_files:
            extension = filename[filename.rfind(".") + 1 :]
            for language in LanguageFinder.all():
                if extension in language.extensions:
                    problem.language = language
                    problem.language.ext = extension
                    return language
        raise Exception("Solution not found!")

    @staticmethod
    def set_problem_name(problem: Problem) -> str:
        filename = File.get_all_files(directory=problem.path, ext=problem.language.ext)[
            0
        ]
        problem.name = filename[: filename.rfind(".")]
        return problem.name



from ..helpers.model.finders.language_finder import LanguageFinder
from ..models.language import Language
from ..helpers.terminal.single_process import SingleProcess
from ..helpers.command.command_helper import CommandHelper
from ..executors.executor import Executor
from pylib_0xe.file.file import File


class CppExecutor(Executor):
    @staticmethod
    def language() -> Language:
        return LanguageFinder.by_abbreviation("cpp")

    def execute(self):
        # compile the problem
        # run the problem give input to it
        self.pipeline(self.compile, self.run)

    def compile(self):
        self.attach_log(f"compiling {self.problem.name}...")
        command = CommandHelper.cpp_compile_command()
        command = command.format(self.problem.full_name)
        out, err = SingleProcess(command).run().communicate()
        if len(err) > 0:
            # we have an error
            self.handle_error(err)
        self.handle_output(out)

    def run(self):
        self.attach_log(f"running {self.problem.name}...")
        # command = CommandHelper.cpp_run_command()
        # command = command.format(self.problem.full_input_name)
        input_file = File.read_file(self.problem.full_input_name)
        input_file += "\r\n"
        out, err = (
            SingleProcess(CommandHelper.cpp_run_command()).run().communicate(input_file)
        )
        if len(err) > 0:
            # we have an error
            self.handle_error(err)
        self.handle_output(out)



from __future__ import annotations
import os
from dataclasses import dataclass
from ..helpers.file.file_helper import FileHelper
from ..helpers.model.problem_helper import ProblemHelper
from ..models.contest import Contest


@dataclass
class ContestGenerator:
    contest: Contest

    def read_template(self) -> ContestGenerator:
        self.template = FileHelper.read_template(
            provider=self.contest.provider, language=self.contest.language
        )
        return self

    def build_path(self) -> ContestGenerator:
        self.contest.path = os.path.join(self.contest.provider.name, self.contest.name)
        print(f"generating the following contest\n\n{self.contest}")
        return self

    def generate_solutions(self) -> ContestGenerator:
        # generating solutions
        # solution file extension
        for i in range(self.contest.problem_cnt):
            problem_name = ProblemHelper.create_problem_name(i, self.contest.name_type)
            # {problem_name}
            # {problem_number}
            # {total_problems}
            # {provider_site}
            # {contest_name}
            solution = FileHelper.template_filler(
                self.template,
                chr(ord("A") + i),
                str(i + 1),
                str(self.contest.problem_cnt),
                self.contest.provider.name,
                self.contest.name,
            )
            # creating the solution
            FileHelper.write_file(
                self.contest.path,
                problem_name,
                f"{problem_name}.{self.contest.language.ext}",
                file=solution,
            )
        return self

    def generate_inputs(self) -> ContestGenerator:
        for i in range(self.contest.problem_cnt):
            problem_name = ProblemHelper.create_problem_name(i, self.contest.name_type)
            # creating input file
            FileHelper.write_file(
                self.contest.path,
                problem_name,
                f"{problem_name}.in",
                file="",
            )
        return self



from dataclasses import dataclass, field
from pylib_0xe.data.data_transfer_object import DataTransferObject


@dataclass
class Provider(DataTransferObject):
    """
    Represents a provider.
    """

    name: str
    abbreviations: list[str] = field(default_factory=list)

    @staticmethod
    def name_mapper(name: str) -> str:
        """
        Mapper function for provider name.
        """
        return name.lower()

    @staticmethod
    def abbreviations_mapper(abbr_list: list[str]) -> list[str]:
        """
        Mapper function for provider abbreviations.
        """
        return [abbr.lower() for abbr in abbr_list]



import os
from ...models.language import Language
from ...models.provider import Provider
from ...helpers.folder.folder_helper import FolderHelper
from ...helpers.config.local_config import LocalConfig
from pylib_0xe.file.file import File
from pylib_0xe.buffer_io.buffer_reader import BufferReader
from pylib_0xe.buffer_io.string_buffer import StringBuffer


class FileHelper:
    @staticmethod
    def extract_extension(path: str) -> str:
        index = path.rfind(".")
        if index == -1:
            return ""
        return path[index + 1 :]

    @staticmethod
    def write_file(*paths: str, **kwargs: str) -> None:
        # extracting filename
        filename = paths[-1]
        paths = paths[:-1] if len(paths) > 1 else ()

        # generating directories recursively
        FolderHelper.create_path(*paths)

        # append the file to the path
        path = os.path.join(*paths, filename)

        # write the file
        file = kwargs["file"]
        try:
            File.write_file(file_path=path, data=file)
        except Exception as e:
            print(f"could'nt create [{path}] file")
            print(e)

    @staticmethod
    def read_template(provider: Provider, language: Language) -> str:
        paths: list[str] = LocalConfig.read(
            f"templates.{provider.name}.{language.name}"
        )
        return File.read_file(os.path.join(*paths))

    @staticmethod
    def template_filler(template: str, *args: str) -> str:
        reader = BufferReader(StringBuffer(template))
        taken = 0
        page = ""
        while not reader.end_of_buffer():
            line = reader.next_line()
            if taken < len(args):
                cnt = line.count("{}")
                if cnt > 0:
                    try:
                        line = line.format(*args[taken : taken + cnt])
                        taken += cnt
                    except Exception as e:
                        print(e)
            page += line
        return page



from ....helpers.config.local_config import LocalConfig
from ....models.language import Language


class LanguageFinder:
    @staticmethod
    def by_name(name: str) -> Language:
        filtered = LanguageFinder.filter_languages(
            lambda language: name == language.name
        )
        return LanguageFinder.get_single_language(filtered, name)

    @staticmethod
    def by_abbreviation(abbreviation: str) -> Language:
        filtered = LanguageFinder.filter_languages(
            lambda language: abbreviation in language.abbreviations
        )
        return LanguageFinder.get_single_language(filtered, abbreviation)

    @staticmethod
    def all() -> list[Language]:
        languages = LocalConfig.read("languages")
        return [
            Language.from_dict({"name": language, **properties})
            for language, properties in languages.items()
        ]

    @staticmethod
    def default() -> Language:
        problem_language = LocalConfig.read("problem.language")
        return LanguageFinder.by_abbreviation(problem_language)

    @staticmethod
    def filter_languages(predicate) -> list[Language]:
        return list(filter(predicate, LanguageFinder.all()))

    @staticmethod
    def get_single_language(filtered: list[Language], search_criteria: str) -> Language:
        if not filtered:
            raise Exception(f"No valid language found by {search_criteria}")
        return filtered[0]



from __future__ import annotations
from dataclasses import dataclass, field
from pylib_0xe.argument.argument_parser import ArgumentParser


@dataclass
class ArgHelper:
    attributes: list[str]
    data: dict[str, str] = field(default_factory=dict)
    prefix: str = "--"

    def show_valid_options(self):
        print(f"available options are: {self.attributes}")
        return self

    def done(self):
        exit(0)

    def check_help(self, help_keyword="help") -> ArgHelper:
        for key, _ in ArgumentParser.get_options(option_prefix=self.prefix).items():
            key = key.lower()
            if key == help_keyword:
                self.show_valid_options().done()

        return self

    def check_args(self, skipps: list[str] = []) -> ArgHelper:
        for key, value in ArgumentParser.get_options(option_prefix=self.prefix).items():
            key = key.lower()
            if key in skipps:
                continue
            if key not in self.attributes:
                print(f"Error: available options are: {self.attributes}")
                exit(0)
            if value == "":
                continue
            self.data[key] = value
        return self

    def must_include(self, *args) -> ArgHelper:
        for arg in args:
            if not ArgumentParser.is_option(arg) or self.data[arg] == "":
                print(f"Error: set the {self.prefix}{arg}")
                exit(0)
        return self

    def get_data(self) -> dict:
        return self.data



from __future__ import annotations
from ..helpers.config.local_config import LocalConfig
from ..helpers.terminal.arg_helper import ArgHelper
from ..generators.contest_generator import ContestGenerator
from ..models.contest import Contest


class ContestMediator:
    def read_configs(self) -> ContestMediator:
        self.attributes = Contest.modifiables()
        self.contest_data = LocalConfig.read("contest")
        return self

    def read_args(self) -> ContestMediator:
        self.contest_data = (
            ArgHelper(attributes=self.attributes, data=self.contest_data, prefix="--")
            .check_help()
            .check_args(skipps=[])
            .must_include("name")
            .get_data()
        )
        return self

    def generate(self) -> ContestMediator:
        contest: Contest = Contest.from_dict(self.contest_data)
        ContestGenerator(
            contest=contest
        ).read_template().build_path().generate_solutions().generate_inputs()
        return self

    def closure(self) -> ContestMediator:
        print("DONE")
        return self



from ...types.languages import Languages


class LanguageHelper:
    @staticmethod
    def extension_mapper(language: Languages) -> str:
        # mapping language to it's corresponding extension
        if language is Languages.CPP:
            return "cpp"
        if language is Languages.PYTHON:
            return "py"
        return "cpp"



from .src.mediators.contest_mediator import ContestMediator
from .src.mediators.execution_mediator import ExecutionMediator
from .src.mediators.problem_mediator import ProblemMediator
from .src.mediators.init_mediator import InitMediator
from .src.helpers.terminal.cat_helper import CatHelper
from pylib_0xe.argument.argument_parser import ArgumentParser

ARG_PREFIX = "--"


def handle_init():
    """Handle initialization."""
    InitMediator().read_configs().create_local_config().create_templates().closure()


def handle_run():
    """Handle program execution."""
    ExecutionMediator().read_configs().read_args().extract_language().extract_name().execute()


def handle_problem():
    """Handle problem solving."""
    ProblemMediator().read_configs().read_args().generate().closure()


def handle_contest():
    """Handle contest management."""
    ContestMediator().read_configs().read_args().generate().closure()


def handle_help():
    """Handle help command"""
    CatHelper.meow()


def main():
    """Main function."""
    if ArgumentParser.is_option("init", option_prefix=ARG_PREFIX):
        handle_init()
    elif ArgumentParser.is_option("run", option_prefix=ARG_PREFIX):
        handle_run()
    elif ArgumentParser.is_option("problem", option_prefix=ARG_PREFIX):
        handle_problem()
    elif ArgumentParser.is_option("name", option_prefix=ARG_PREFIX):
        handle_contest()
    else:
        handle_help()


if __name__ == "__main__":
    main()



from ...helpers.config.local_config import LocalConfig


class CommandHelper:
    @staticmethod
    def cpp_compile_command():
        return LocalConfig.read("executor.commands.cpp.compile")

    @staticmethod
    def cpp_run_command():
        return LocalConfig.read("executor.commands.cpp.run")

    @staticmethod
    def python_run_command():
        return LocalConfig.read("executor.commands.python.run")
